function M = ApproximateM(zeta,param,Mlag)
%This function provides an approximation to the endogenous weight matrix
%   Mlag = pervious period spatial weight matrix
%   param = parameters of the model
%   zeta  = individual specific location variables


n      = size(zeta,1);
d2s    = zeros(n,n);
d3s    = zeros(n,n);
l      = zeros(n,n);
M      = zeros(n,n);
kappax = param.kappax;
appMe  = param.appMe;
xi     = param.xi;
xid    = param.xid;
xil    = param.xil;

if isempty(Mlag) % there is no prior weight matrix
    for i = 1:n
        for j = i+1:n
            sij      = (EuclidNorm((zeta(j,:) -zeta(i,:))') < kappax/2); 
            switch appMe
                case 'smooth'
                    d2s(i,j) = exp(-xi*EuclidNorm((zeta(j,:) -zeta(i,:))'))*sij;                    
                case 'binary'
                    d2s(i,j) = (xi*EuclidNorm((zeta(j,:) -zeta(i,:))')<1)*sij;
                case 'random'
                    d2s(i,j)     = (rand < LogisticCDF( -xi*EuclidNorm((zeta(j,:) -zeta(i,:))) ) );
            end
            d2s(j,i) = d2s(i,j);
        end
        d2sum = d2s(i,:)*ones(n,1);
        if d2sum == 0
           M(i,:) = zeros(1,n);
        else
            M(i,:) = d2s(i,:)/(d2sum);
        end
    end
else % user supplied lagged spatial weight matrix
    % Recover adjacency matrix from row-normalized matrix
    d2s  = (Mlag~=0) + zeros(n,n); %note that adding a matrix of zeros converts d2s from locical to double without changing the content of the matrix
    for i=1:n
        for j = i+1:n
            sij      = (EuclidNorm((zeta(j,:) -zeta(i,:))') < kappax/2); 
            l(i,j)   = d2s(i,:)*d2s(:,j);
            switch appMe
                case 'smooth'
                    d3s(i,j) = exp(-xi*EuclidNorm((zeta(j,:) -zeta(i,:))'))*LogisticCDF(xid*d2s(i,j))*LogisticCDF(xil*l(i,j))*sij;
                case 'binary'
                    d3s(i,j) = (xi*EuclidNorm((zeta(j,:) -zeta(i,:))')<1)*LogisticCDF(xid*d2s(i,j))*LogisticCDF(xil*l(i,j))*sij;
                case 'random'
                    d3s(i,j)     = (rand < LogisticCDF( -xi*EuclidNorm((zeta(j,:) -zeta(i,:))') + xid*d2s(i,j) + xil*l(i,j) ));
            end
            d3s(j,i) = d3s(i,j);
        end
        d3sum = d3s(i,:)*ones(n,1);
        if d3sum == 0
           M(i,:) = zeros(1,n);
        else
            M(i,:) = d3s(i,:)/(d3sum);
        end
    end
end

