classdef DataClassDefinitions < handle
    properties
        Param = struct('n',[],'amu',[],'kappa',[],'azeta',[],'a0',[],'beta1',[],'beta2',[],...
                       'beta3',[],'beta4',[],'gamma',[],'factor',[],'LocalID',[],'lambda',...
                       [],'d',[],'type',[],'endogenous',[],'xi',[],'xid',[],'xil',[],'kappax',[],'factoralph',...
                       [],'factorsig', [], 'zetadep',[],'zetasig',[],'mu_gam',[],'sigmav',[],'staticM',[],...
                       'appMe',[],'projectA',[]);
        yDat
        ZDat
        DDat
        MDat  
        WDat
        HDat
        H1Dat
        H2Dat        
        HADat
        HstarDat        
        H1starDat        
        H2starDat  
        
        HdynDat
        HAdynDat
        HstardynDat
        Hdyn1Dat
        HAdyn1Dat
        Hstardyn1Dat

        Hdyn2Dat   
        HAdyn2Dat  
        Hstardyn2Dat

        AmFDat
        AmFstarDat
        MstarDat
        VmFDat
        VmFstarDat
        yplusDat
        yMplusDat
        WplusDat
        ZplusDat
        zplusDat
        MZplusDat
        W1plusDat
        W2plusDat        
        yMLplusDat        
        zetaDat
        delta
        lambda
        lambda_first
        lambda_oneIt        
        beta
        beta_first
        beta_oneIt
        delta_1
        delta_2
        mEig
        R2Dat
        dynamic
        
    end
    methods 
        function ySim = DataClassDefinitions(Param)
            ySim.Param      = Param;
        end
                function ySim = OLS(ySim)
            yplus           = ySim.yplusDat;
            Wplus           = ySim.WplusDat;
            if ySim.dynamic
                T          = ySim.Param.T - 2;
            else
                T          = ySim.Param.T - 1;
            end
            n               = ySim.Param.n;
            dimW            = size(Wplus,1)/n;

            ySim.delta      = Estimator_OLS(yplus,Wplus,n,T,dimW);
            ySim.lambda     = ySim.delta(1,1);
            ySim.beta       = ySim.delta(2:end,1);
            ySim.Param.type ='OLS';
        end
        function ySim = IV(ySim,InstrumentType,endogenous)
            yplus        = ySim.yplusDat;
            Wplus        = ySim.WplusDat;
            Myplus       = ySim.yMplusDat;
            dynamicM     = ySim.dynamic;
            
            ySim.Param.type = SetModelTypeIV(InstrumentType);
            
            if dynamicM
                T        = ySim.Param.T - 2;
            else
                T        = ySim.Param.T - 1;
            end
            
            
            if dynamicM && ~endogenous
                switch InstrumentType
                    case 0
                        H               = ySim.HdynDat;
                    case 1
                        H               = ySim.Hdyn1Dat;
                    case 2
                        H               = ySim.Hdyn2Dat;
                end
            elseif dynamicM && endogenous
                H        = ySim.HstardynDat;
            elseif ~dynamicM && ~endogenous
                switch InstrumentType
                    case 0
                        H               = ySim.HDat;
                    case 1
                        H               = ySim.H1Dat;
                    case 2
                        H               = ySim.H2Dat;
                end
            elseif ~dynamicM && endogenous
                switch InstrumentType
                    case 0
                        H        = ySim.HstarDat;
                    case 1
                        H        = ySim.H1starDat;
                    case 2
                        H        = ySim.H2starDat;
                end
            end
            
            
            n               = ySim.Param.n;
            
            [ySim.delta,F]  = Estimator_IV( yplus,Myplus,Wplus,H,n,T );
            
            ySim.lambda = ySim.delta(1,1);
            ySim.beta   = ySim.delta(2:end,1);
            %               ySim.R2Dat  = R2;
            ySim.R2Dat  = F;
            
        end
        function ySim = GMM_OptComp(ySim,InstrumentType)
            y          = ySim.yDat;
            M          = ySim.MDat;
            Z          = ySim.ZDat;
            
            
            n          = ySim.Param.n;
            AmF        = ySim.AmFDat;
            VmF        = ySim.VmFDat;
            
            T          = ySim.Param.T;
           
            efficient  = true;
            endogenous = false;
            ySim.Param.type = SetModelType(InstrumentType,efficient,endogenous);
            
            dynamicM   = ySim.dynamic;
            
            if dynamicM
                switch InstrumentType
                    case 0
                        H               = ySim.HdynDat;
                    case 1
                        H               = ySim.Hdyn1Dat;
                    case 2
                        H               = ySim.Hdyn2Dat;
                end
            else
                switch InstrumentType
                    case 0
                        H               = ySim.HDat;
                    case 1
                        H               = ySim.H1Dat;
                    case 2
                        H               = ySim.H2Dat;
                end
            end
            [lambda_hat,beta_hat,lambda_f,beta_f] = Estimator_GMMFactorOptComp(y,Z,M,H,AmF,VmF,dynamicM,n,T);
            ySim.lambda_first = lambda_f;
            ySim.beta_first   = beta_f;
            ySim.lambda       = lambda_hat;
            ySim.beta         = beta_hat;
        end
        function ySim = GMM_Simplified_Modified(ySim,InstrumentType)
            y          = ySim.yDat;
            M          = ySim.MDat;
            Z          = ySim.ZDat;
            
            
            n          = ySim.Param.n;
            AmF        = ySim.AmFDat;
            VmF        = ySim.VmFDat;
            
            T          = ySim.Param.T;
           
            efficient  = true;
            endogenous = false;
            ySim.Param.type = SetModelType(InstrumentType,efficient,endogenous);
            
            dynamicM   = ySim.dynamic;
            
            if dynamicM
                switch InstrumentType
                    case 0
                        H               = ySim.HdynDat;
                    case 1
                        H               = ySim.Hdyn1Dat;
                    case 2
                        H               = ySim.Hdyn2Dat;
                end
            else
                switch InstrumentType
                    case 0
                        H               = ySim.HDat;
                    case 1
                        H               = ySim.H1Dat;
                    case 2
                        H               = ySim.H2Dat;
                end
            end
            [lambda_hat,beta_hat,lambda_f,beta_f] = Estimator_GMMFactor(y,Z,M,H,AmF,VmF,dynamicM,n,T);
            ySim.lambda_first = lambda_f;
            ySim.beta_first   = beta_f;
            ySim.lambda       = lambda_hat;
            ySim.beta         = beta_hat;
        end
        function ySim = GMMFixedF(ySim,efficient,dynamic,InstrumentType,endogenous) % imposes that all factors are = 1 for the dynamic model without other covs but estimate variances
            y          = ySim.yDat;
            Z          = ySim.ZDat;
            M          = ySim.MDat;
            T          = ySim.Param.T;
            n          = ySim.Param.n;
            dynamicM   = ySim.dynamic;
            % Instrument selection depending on whether the model is dynamic and/or is believed to have endogenous M
            ySim.Param.type = SetModelType(InstrumentType,efficient,endogenous);
            
            if dynamicM && ~endogenous
                switch InstrumentType
                    case 0
                        H               = ySim.HdynDat;
                    case 1
                        H               = ySim.Hdyn1Dat;
                    case 2
                        H               = ySim.Hdyn2Dat;
                end
                AmF      = ySim.AmFDat;
                VmF      = ySim.VmFDat;
            elseif dynamicM && endogenous
                H        = ySim.HstardynDat;
                AmF      = ySim.AmFstarDat;
                VmF      = ySim.VmFstarDat;
            elseif ~dynamicM && ~endogenous
                switch InstrumentType
                    case 0
                        H               = ySim.HDat;
                    case 1
                        H               = ySim.H1Dat;
                    case 2
                        H               = ySim.H2Dat;
                end
                AmF      = ySim.AmFDat;
                VmF      = ySim.VmFDat;
            elseif ~dynamicM && endogenous
                switch InstrumentType
                    case 0
                        H        = ySim.HstarDat;
                    case 1
                        H        = ySim.H1starDat;
                    case 2
                        H        = ySim.H2starDat;
                end
                AmF      = ySim.AmFstarDat;
                VmF      = ySim.VmFstarDat;
            end
            
            
            [lambda_hat,beta_hat,lambda_f,beta_f] = Estimator_GMMFixed( y,Z,H,M,AmF,VmF,dynamic,n,T );    
            ySim.lambda_first = lambda_f;
            ySim.beta_first   = beta_f;
            
            ySim.lambda       = lambda_hat;
            ySim.beta         = beta_hat;
            ySim.dynamic      = dynamic;
        end
    end
end
     
                
         
    
           