function [f,sigma,y_plus,W1_plus,W2_plus]= EstimateFactors(y,ZIn,M,H,AmF,VmF,dynamic)

%Notation: XT = variable X measured at time T, XT1 = variable X measured at
%T-1; XT1_2 = variable X, measured at T-1, raised to the power 2. 
% Modification refers to reversing the regression 

n         = size(y,1);
n2        = n^2;
T         = size(y,2);
f         = zeros(T,1);
sigma     = zeros(T,1);
phi       = zeros(T-1,1);
Pi        = zeros(T-1,T);
Pi_tilde  = zeros(T-1,T);
My        = zeros(n,T);
My1       = zeros(n,T);
Z         = zeros(2*n,T);  %dimension is model specific for AR(1) type panel

y_plus    = zeros(n,T);
dimH      = size(H,1)/n;
f(T,1)    = 1;   %normalize factor 
sigma(T,1)= 1;   %normalize variance


if dynamic
    W1_plus   = zeros(n,T);
    W2_plus   = zeros(2*n,T);
    lb        = 2; %need two initial periods to compute IV
else
    W1_plus   = zeros(2*n,T);
    W2_plus   = zeros(n,T);
    lb        = 1; %need one initial period for quasi differencing
end


%construct My

for t=1:T
    My(:,t)= reshape(M(:,t),n,n)*y(:,t);
    if ~dynamic
        Z(:,t) = [ZIn(:,t);reshape(M(:,t),n,n)*ZIn(:,t)];
    end
    if t>1
       if dynamic
            Z(:,t)   = [y(:,t-1);reshape(M(:,t),n,n)*y(:,t-1)];
       end
       My1(:,t) = reshape(M(:,t),n,n)*y(:,t-1);
    end    
end

%Start Recursion at T
 MT     = reshape(M(:,T),n,n);
 MT1    = reshape(M(:,T-1),n,n);

if dynamic
    A1       = reshape(AmF(1:n2,T-2),n,n);
    A2       = reshape(AmF(n2+1:2*n2,T-2),n,n);
    A3       = reshape(AmF(2*n2+1:3*n2,T-2),n,n);
    A4       = reshape(AmF(3*n2+1:4*n2,T-2),n,n);
    Va       = reshape(VmF(:,T-2),4,4);
    hT       = reshape(H(:,T-2),n,dimH);
    W2T      = [y(:,T),MT*y(:,T),y(:,T-1),MT*y(:,T-1)];
    W1T      = [MT1*y(:,T-1),y(:,T-2),MT1*y(:,T-2)];
else
    A1       = reshape(AmF(1:n2,T-1),n,n);
    A2       = reshape(AmF(n2+1:2*n2,T-1),n,n);
    A3       = reshape(AmF(2*n2+1:3*n2,T-1),n,n);
    A4       = reshape(AmF(3*n2+1:4*n2,T-1),n,n);
    Va       = reshape(VmF(:,T-1),4,4);
    hT       = reshape(H(:,T-1),n,dimH);
    W2T    = [y(:,T),MT*y(:,T),Z(1:n,T),Z(n+1:2*n,T)];
    W1T    = [MT1*y(:,T-1),Z(1:n,T),Z(n+1:2*n,T)];    
end


yT1    = y(:,T-1);
yT     = y(:,T);

if dimH<4
    if dynamic
        hT = [hT,MT*MT*MT*y(:,T-1)];
    else
        hT = [hT,MT*MT*MT*Z(1:n,T)];
    end
end
Ph     = hT/(hT'*hT)*hT';
Qh     = eye(n)-W2T/(W2T'*Ph*W2T)*W2T'*Ph;


oldoptions = optimoptions(@fminunc); %save old options
options    = optimoptions(@fminunc,'Algorithm','quasi-newton','MaxIterations',1,'Display','off'); % reduce Iterations to just one
lambda0        = zeros(3,1);
[lambda,~]     = fminunc(@(lambda)mq_t_factor(lambda,yT1,W1T,Qh,A1,A2,A3,A4,Va),lambda0,options);
optimoptions(@fminunc,oldoptions);


yT_tilde       = yT  - W2T(:,2:end) * lambda;
yT1_tilde      = yT1 - W1T * lambda;

f(T-1,1)       = yT_tilde'*Ph*yT1_tilde/(yT_tilde'*Ph*yT_tilde);


uT1_plus       = yT1_tilde - f(T-1,1)*yT_tilde;
sigma(T-1,1)   = uT1_plus'*uT1_plus/n-f(T-1,1)^2;
if sigma(T-1,1)<0 
    sigma(T-1,1) = 1; %=sigma(T,1).
end
phi(T-1,1)     = (f(T-1,1)^2/sigma(T-1,1))+1;
phi(T,1)       = 1;

Pi(T-1,T-1)    = sqrt(phi(T,1)/phi(T-1,1))/sqrt(sigma(T-1,1));
Pi(T-1,T)      = -f(T-1,1)*f(T,1)*sqrt(phi(T,1)/phi(T-1,1))/(phi(T,1)*sqrt(sigma(T-1,1))*sigma(T,1));

Pi_tilde(T-1,T-1)    = 0;
Pi_tilde(T-1,T)      = 1;


% compute data for period T-1 linear  quadratic moments
yT_plus        = y*Pi(T-1,:)';
ZT_plus        = Z*Pi(T-1,:)';
yT1_plus       = [zeros(n,1),y(:,1:T-1)]*Pi(T-1,:)';
MyT_plus       = My*Pi(T-1,:)';
MyT1_plus      = My1*Pi(T-1,:)';

if dynamic
    W1T_plus       = MyT_plus;
    W2T_plus       = [yT1_plus,MyT1_plus];
else
    W1T_plus       = [MyT_plus, ZT_plus(1:n,:)];
    W2T_plus       = ZT_plus(n+1:2*n,:);
end

% store data for period T-1 linear quadratic criterion
y_plus(:,T)    = yT_plus;
W1_plus(:,T)   = vec(W1T_plus);
W2_plus(:,T)   = vec(W2T_plus);


for t = T-2:-1:lb  %count down from T-1 to 1
    for s = t+1:T %construct forward filtering matrix
        Pi_tilde(t,s)  = f(s,1)/(phi(t+1,1)*sigma(s,1));
    end
    Mt     = reshape(M(:,t),n,n);
    
    yt_dag   = y  * Pi_tilde(t,:)';
    yt1_dag  = [zeros(n,1),y(:,2:end)] * Pi_tilde(t,:)';
    yt_dagb  = My * Pi_tilde(t,:)';
    yt1_dagb = [zeros(n,1),My(:,2:end)] * Pi_tilde(t,:)';    
    Zt_dag   = Z  * Pi_tilde(t,:)';
    yt       = y(:,t);
    
    if dynamic
        Mt1    = reshape(M(:,t-1),n,n);
        ht     = reshape(H(:,t-1),n,dimH);
        W2t    = [yt_dag,yt_dagb,yt1_dag,yt1_dagb];
        W1t    = [MT*y(:,t),y(:,t-1),Mt1*y(:,t-1)];        
    else
        ht     = reshape(H(:,t),n,dimH);
        W1t    = [My(:,t),reshape(Z(:,t),n,2)];
        W2t    = [yt_dag,yt_dagb,reshape(Zt_dag,n,2)];        
    end

    if dimH<4
        if dynamic
            ht = [ht,Mt*Mt*Mt*y(:,t-1)];
        else
            ht = [ht,Mt*Mt*Mt*Z(1:n,t)];
        end
    end
    Ph     = ht/(ht'*ht)*ht';
   
    yt_tilde       = yt     - W1t * lambda;
    ytdag_tilde    = yt_dag - W2t(:,2:end) * lambda;

    f(t,1)         = yt_tilde'*Ph*ytdag_tilde/(ytdag_tilde'*Ph*ytdag_tilde);

    ut_plus        = yt_tilde - f(t,1)*ytdag_tilde;
    sigma(t,1)     = ut_plus'*ut_plus/n-f(t,1)^2/phi(t+1,1);
    if sigma(t,1)<0
        sigma(t,1)=sigma(t+1,1);
    end
    phi(t,1)       = f(t,1)^2/sigma(t,1)+phi(t+1,1);
    
    Pi(t,t)        = sqrt(phi(t+1,1)/phi(t,1)/sigma(t,1));
    for s = t+1:T  % construct forward filtering matrix 
        Pi(t,s)    = -f(t,1)*f(s,1)*sqrt(phi(t+1,1)/phi(t,1))/(phi(t+1,1)*sqrt(sigma(t,1)*sigma(s,1)));
    end
    yt_plus        = y*Pi(t,:)';
    Zt_plus        = Z*Pi(t,:)';
    yt1_plus       = [zeros(n,1),y(:,1:T-1)]*Pi(t,:)';
    Myt_plus       = My*Pi(t,:)';
    Myt1_plus      = My1*Pi(t,:)';
    
    if dynamic
        W1t_plus       = Myt_plus;
        W2t_plus       = [yt1_plus,Myt1_plus];
    else
        W1t_plus       = [Myt_plus,Zt_plus(1:n,:)];
        W2t_plus       = Zt_plus(n+1:2*n,:);
    end        
    
    %store data for period t linear quadratic criterion
    y_plus(:,t+1)    = yt_plus;
    W1_plus(:,t+1)   = vec(W1t_plus);
    W2_plus(:,t+1)   = vec(W2t_plus);
end
if dynamic
    y_plus    = y_plus(:,3:end);
    W1_plus   = W1_plus(:,3:end);
    W2_plus   = W2_plus(:,3:end);
else
    y_plus    = y_plus(:,2:end);
    W1_plus   = W1_plus(:,2:end);
    W2_plus   = W2_plus(:,2:end);
end    
return








