function [lambda,beta] = FixedFactorLinQuadGMM(A,P,Q,V,y_plus,W1_plus,W2_plus,lambda0,dynamic)
% Computes initial step GMM estimator as a consistant starting point into full
% optimization routine
n = size(y_plus,1);
T = size(y_plus,2);

n1      = size(W1_plus,1);
n2      = size(W2_plus,1);

if mod(n1,n)==0
    dim1 = n1/n;
else
    error('Dimension error FixedFactorLinQuadGMM');
end
if mod(n2,n)==0
    dim2 = n2/n;
else
    error('Dimension error FixedFactorLinQuadGMM');
end

% find minimizers of concentrated quadratic criterion

oldoptions = optimoptions(@fminunc); %save old options
options    = optimoptions(@fminunc,'Algorithm','quasi-newton','MaxIterations',100,'Display','off');
if dynamic
    [lambda1,~] = fminunc(@(lambda)mqff(lambda,A,Q,V,y_plus,W1_plus),lambda0(1:dim1,1),options);
else
    [lambda1,~] = fminunc(@(lambda)mqff(lambda,A,Q,V,y_plus,W1_plus),lambda0(1:dim1,1),options);
end
optimoptions(@fminunc,oldoptions);

% Compute remaining parameters from linear IV

lambdalin     = lambda1;
Denominator   = 0;
Numerator     = 0;
for t = 1:T
  if dynamic    
      W1t_plus    = reshape(W1_plus(:,t),n,dim1);  
      W2t_plus    = reshape(W2_plus(:,t),n,dim2);
  else
      W1t_plus    = reshape(W1_plus(:,t),n,dim1);
      W2t_plus    = reshape(W2_plus(:,t),n,dim2);
  end


  yt_plus     = y_plus(:,t);  
  Pht         = reshape(P(:,t),n,n);
  Denominator = Denominator + W2t_plus'*Pht*W2t_plus;
  Numerator   = Numerator   + W2t_plus'*Pht*(yt_plus - W1t_plus*lambdalin);    
end

beta2  = Denominator\Numerator;

lambda = lambda1(1,1);
beta   = [lambda1(2:dim1,1);beta2];

return