function [y_plus,My_plus,My1_plus,W_plus,W1_plus,W2_plus,Z_plus,z_plus]= ForwardDiff(y,Z,M,sigma,dynamic)

%Notation: XT = variable X measured at time T, XT1 = variable X measured at
%T-1; XT1_2 = variable X, measured at T-1, raised to the power 2. 

%This routine computes forward differences for a given vector of sigma (can
%be all equal to 1 and factors normalized to all ones.

n         = size(y,1);
T         = size(y,2);
f         = ones(T,1);    %set all factors equal to 1
phi       = zeros(T-1,1);
Pi        = zeros(T-1,T);
Pi_tilde  = zeros(T-1,T);
My        = zeros(n,T);
MZ        = zeros(n,T);
My1       = zeros(n,T);
Zd        = zeros(2*n,T);  %dimension is model specific for AR(1) type panel
W_plus    = zeros(3*n,T);


if dynamic
    W1_plus   = zeros(n,T);
    W2_plus   = zeros(2*n,T);    
else
    W1_plus   = zeros(n,T);
    W2_plus   = zeros(2*n,T);
end


    
y_plus    = zeros(n,T);


%construct My

for t=1:T
    My(:,t)= reshape(M(:,t),n,n)*y(:,t);
    MZ(:,t)= reshape(M(:,t),n,n)*Z(:,t);
    if t>1
       Zd(:,t)   = [y(:,t-1);reshape(M(:,t),n,n)*y(:,t-1)];
       My1(:,t) = reshape(M(:,t),n,n)*y(:,t-1);
    end    
end

phi(T-1,1)     = (f(T-1,1)^2/sigma(T-1,1))+1;

Pi(T-1,T-1)    = sqrt(1/(phi(T-1,1)));
Pi(T-1,T)      = -f(T-1,1) * sqrt(1/phi(T-1,1));

Pi_tilde(T-1,T-1)    = 0;
Pi_tilde(T-1,T)      = 1;


% compute data for period T-1 linear  quadratic moments
yT_plus        = y*Pi(T-1,:)';
ZT_plus        = Z*Pi(T-1,:)';


yT1_plus       = [zeros(n,1),y(:,1:T-1)]*Pi(T-1,:)';

MyT_plus       = My*Pi(T-1,:)';
MZT_plus       = MZ*Pi(T-1,:)';
MyT1_plus      = My1*Pi(T-1,:)';


if dynamic
    W1T_plus       = MyT_plus;
    W2T_plus       = [yT1_plus,MyT1_plus];
else
    W1T_plus       = MyT_plus;
    W2T_plus       = [ZT_plus,MZT_plus];
end

% store data for period T-1 linear quadratic criterion
y_plus(:,T)    = yT_plus;
yM_plus(:,T)   = MyT_plus;
yM1_plus(:,T)  = MyT1_plus;
W1_plus(:,T)   = vec(W1T_plus);
W2_plus(:,T)   = vec(W2T_plus);
W_plus(:,T)    = [W1_plus(:,T);W2_plus(:,T)];
Z_plus(:,T)    = vec([ZT_plus,MZT_plus]);
z_plus(:,T)    = ZT_plus;



%determine how many time periods to use. If model is dynamic then need to
%account for two lags to form covariates and instruments 
if dynamic
    lb = 2;
else
    lb = 1;
end

for t = T-2:-1:lb  %count down from T-1 to 1
    for s = t+1:T %construct forward filtering matrix
        Pi_tilde(t,s)  = f(s,1)/(phi(t+1,1)*sigma(s,1));
    end

    phi(t,1)       = f(t,1)^2/sigma(t,1)+phi(t+1,1);
    
    Pi(t,t)        = sqrt(phi(t+1,1)/phi(t,1)/sigma(t,1));
    for s = t+1:T  %construct forward filtering matrix - [GK:was T-1, but probably should be T, check]
        Pi(t,s)    = -f(t,1)*f(s,1)*sqrt(phi(t+1,1)/phi(t,1))/(phi(t+1,1)*sqrt(sigma(t,1)*sigma(s,1)));
    end
    yt_plus        = y*Pi(t,:)';
    yt1_plus       = [zeros(n,1),y(:,1:T-1)]*Pi(t,:)';
    zt_plus        = Z*Pi(t,:)';
    Mzt_plus       = MZ*Pi(t,:)';
    Myt_plus       = My*Pi(t,:)';
    Myt1_plus      = My1*Pi(t,:)';
    
    if dynamic
        W1t_plus       = Myt_plus;
        W2t_plus       = [yt1_plus,Myt1_plus];        
    else
        W1t_plus       = Myt_plus;
        W2t_plus       = [zt_plus,Mzt_plus];
    end    
    
    %store data for period t linear quadratic criterion
    y_plus(:,t+1)    = yt_plus;
    yM_plus(:,t+1)   = Myt_plus;
    yM1_plus(:,t+1)  = Myt1_plus;    
    W1_plus(:,t+1)   = vec(W1t_plus);
    W2_plus(:,t+1)   = vec(W2t_plus);
    W_plus(:,t+1)    = [W1_plus(:,t+1);W2_plus(:,t+1)];
    Z_plus(:,t+1)    = vec([zt_plus,Mzt_plus]);
    z_plus(:,t+1)    = zt_plus;

end
if dynamic
    y_plus    = y_plus(:,3:end);
    My_plus   = yM_plus(:,3:end);
    My1_plus  = yM1_plus(:,3:end);    
    W1_plus   = W1_plus(:,3:end);
    W2_plus   = W2_plus(:,3:end);
    W_plus    = W_plus(:,3:end);
    Z_plus    = Z_plus(:,3:end);
    z_plus    = z_plus(:,3:end);

else
    y_plus    = y_plus(:,2:end);
    My_plus   = yM_plus(:,2:end);
    My1_plus  = yM1_plus(:,2:end);        
    W1_plus   = W1_plus(:,2:end);
    W2_plus   = W2_plus(:,2:end);
    W_plus    = W_plus(:,2:end);
    Z_plus    = Z_plus(:,2:end);
    z_plus    = z_plus(:,2:end);
end
    
return








