function [ ulam,ulamh,ulamQ,hh,n,T ] = FullOptimDataPrepFlexFactor( y,W1,W2,h,A,Q,dynamic )
% computes sufficient statistics for criterion of estimated factor model


n         = size(y,1);
numI      = size(h,1)/n;
T1        = size(y,2);
if dynamic
    T         = T1 - 2;
else
    T         = T1 - 1;
end

n1      = size(W1,1);
n2      = size(W2,1);


if mod(n1,n)==0
    dim1 = n1/n;
else
    error('Dimension error mlqFixedF');
end
if mod(n2,n)==0
    dim2 = n2/n;    
else
    error('Dimension error mlqFixedF');
end


d         = 1 + dim1 + dim2;
ulamh     = zeros(T,numI*(1+dim1+dim2)*T1);


%first unpackW quadratic weight matrices
At        = reshape(A(:,T),n^2,4);
At1       = reshape(At(:,1),n,n);
At2       = reshape(At(:,2),n,n);
At3       = reshape(At(:,3),n,n);
At4       = reshape(At(:,4),n,n);
hT        = reshape(h(:,T),n,numI);

QT        = reshape(Q(:,T),n,n);


ulam((T-1)*n+1:T*n,:)      = [y,-unpackW(W1,n,dim1,T1),- unpackW(W2,n,dim2,T1)];
ulamh(T,:)                 = vec(hT'*ulam((T-1)*n+1:T*n,:))';
hh(:,T)                    = vec(hT'*hT);

if rank(At1)==0 && rank(At2)~=0 && rank(At3)~=0
    ulamQ((T-1)*n+1:T*n,:)   = [zeros(n,d*T1),QT*At2*QT*ulam((T-1)*n+1:T*n,:),...
                                QT*At3*QT*ulam((T-1)*n+1:T*n,:),...
                                QT*At4*QT*ulam((T-1)*n+1:T*n,:)];
elseif rank(At1)==0 && rank(At2)==0 && rank(At3)~=0
    ulamQ((T-1)*n+1:T*n,:)   = [zeros(n,2*d*T1),QT*At3*QT*ulam((T-1)*n+1:T*n,:),...
                                QT*At4*QT*ulam((T-1)*n+1:T*n,:)];
elseif rank(At1)==0 && rank(At2)==0 && rank(At3)==0
    ulamQ((T-1)*n+1:T*n,:)   = [zeros(n,3*d*T1),QT*At4*QT*ulam((T-1)*n+1:T*n,:)];
else
    ulamQ((T-1)*n+1:T*n,:)   = [QT*At1*QT*ulam((T-1)*n+1:T*n,:),...
                                QT*At2*QT*ulam((T-1)*n+1:T*n,:),...
                                QT*At3*QT*ulam((T-1)*n+1:T*n,:),...
                                QT*At4*QT*ulam((T-1)*n+1:T*n,:)];
end


for t = T-1:-1:1
    At        = reshape(A(:,t),n^2,4);
    At1       = reshape(At(:,1),n,n);
    At2       = reshape(At(:,2),n,n);
    At3       = reshape(At(:,3),n,n);
    At4       = reshape(At(:,4),n,n); 
    Qt        = reshape(Q(:,t),n,n);
    ht        = reshape(h(:,t),n,numI);
    
    ulam((t-1)*n+1:t*n,:)          = [y, - unpackW(W1,n,dim1,T1), - unpackW(W2,n,dim2,T1)];
    ulamh(t,:)                      = vec(ht'*ulam((t-1)*n+1:t*n,:))';
    hh(:,t)                         = vec(ht'*ht);
    
    if rank(At1)==0 && rank(At2)~=0 && rank(At3)~=0
        ulamQ((t-1)*n+1:t*n,:)   = [zeros(n,d*T1),Qt*At2*Qt*ulam((t-1)*n+1:t*n,:),...
                                         Qt*At3*Qt*ulam((t-1)*n+1:t*n,:),...
                                         Qt*At4*Qt*ulam((t-1)*n+1:t*n,:)];
    elseif rank(At1)==0 && rank(At2)==0 && rank(At3)~=0
        ulamQ((t-1)*n+1:t*n,:)   = [zeros(n,2*d*T1),Qt*At3*Qt*ulam((t-1)*n+1:t*n,:),...
                                         Qt*At4*Qt*ulam((t-1)*n+1:t*n,:)];
    elseif rank(At1)==0 && rank(At2)==0 && rank(At3)==0
        ulamQ((t-1)*n+1:t*n,:)   = [zeros(n,2*d*T1),Qt*At4*Qt*ulam((t-1)*n+1:t*n,:)];
    else
        ulamQ((t-1)*n+1:t*n,:)   = [Qt*At1*Qt*ulam((t-1)*n+1:t*n,:),...
                                    Qt*At2*Qt*ulam((t-1)*n+1:t*n,:),...
                                    Qt*At3*Qt*ulam((t-1)*n+1:t*n,:),...
                                    Qt*At4*Qt*ulam((t-1)*n+1:t*n,:)];
    end
end



