function [Mt,Rt,dt] = GenMt(A,R1,d1,ups,v,zeta,gamma,kappa,a0,azeta,delta_1,delta_2,n,t)
% generates the Mt matrix according to Graham(2016)

upst = reshape(ups(:,t),n,n);
dt   = zeros(n,n);
Mt   = zeros(n,n);

for i=1:n
    zetai  = zeta(i,:);
    Mti    = zeros(1,n);
    dti    = zeros(1,n);
    for j=1:n
        upst(i,j)  = gamma*(v(i,t)+v(j,t))/2 + upst(i,j);
        D2ij       = a0 + azeta*EuclidNorm((zeta(j,:) -zetai)')+ A(i,j)+delta_1*R1(i,j)+delta_2*d1(i,j)+ upst(i,j);
        sij        = (EuclidNorm((zeta(j,:) -zetai)') < kappa/2); 
        Mti(j)     = (D2ij>0)*sij;
        dti(j)     = (D2ij>0)*sij;
    end
    Mt(i,:)  = Mti;
    Mt(i,i)  = 0;                
    Mts      = Mt(i,:)*ones(n,1);   
    dt(i,:)  = Mt(i,:);
    if Mts==0
        Mt(i,:)  = zeros(1,n);
    else
        Mt(i,:)  = Mt(i,:)./(ones(1,n)*Mts);
    end
end
Rt = dt*dt';
end