function [A,P,Q,V] = formatAPQV(w,H,AmF,VmF,dynamic,projectA,Qind,n,T)
%Qind indicates whether projection for quad moments should be used 

dimH      = size(H,1)/n;
dimW      = size(w,1)/n;

T         = T-1;    
n2        = n^2;

Q         = zeros(n^2,T);
A         = zeros(4*n^2,T);
V         = zeros(4^2,T);
P         = zeros(n^2,T);

if dynamic
    hT       = reshape(H(:,T-1),n,dimH);      
    wT       = reshape(w(:,T-1),n,dimW);
else
    hT       = reshape(H(:,T),n,dimH);
    wT       = reshape(w(:,T),n,dimW);
end

PhT      = hT/(hT'*hT)*hT';


QhT      = eye(n)-wT/(wT'*PhT*wT)*wT'*PhT;

A1       = reshape(AmF(1:n2,T),n,n);
A2       = reshape(AmF(n2+1:2*n2,T),n,n);
A3       = reshape(AmF(2*n2+1:3*n2,T),n,n);
A4       = reshape(AmF(3*n2+1:4*n2,T),n,n);
Va       = reshape(VmF(:,T),4,4);

if projectA
    QhTe = eye(n)-PhT;
    A1   = QhTe * A1 * QhTe;
    A2   = QhTe * A2 * QhTe;
    A3   = QhTe * A3 * QhTe;
    A4   = QhTe * A4 * QhTe;
    
    A1   = A1 - diag(diag((A1)));
    A2   = A2 - diag(diag((A2)));
    A3   = A3 - diag(diag((A3)));
    A4   = A4 - diag(diag((A4)));
end

if ~Qind
    QhT      = eye(n);
end

P(:,T)         = vec(PhT);
Q(:,T)         = vec(QhT);
A(:,T)         = vec([vec(A1),vec(A2),vec(A3),vec(A4)]);
V(:,T)         = vec(Va);

if dynamic
    lb = 2;
else
    lb = 1;
end


for t = T-1:-1:lb 
    if dynamic
        ht        = reshape(H(:,t-1),n,dimH);
        wt        = reshape(w(:,t-1),n,dimW);
    else
        ht        = reshape(H(:,t),n,dimH);
        wt        = reshape(w(:,t),n,dimW);
    end
    
    
    Pht       = ht/(ht'*ht)*ht';
    
    Qht       = eye(n)-wt/(wt'*Pht*wt) * wt'*Pht;
    
    At1       = reshape(AmF(1:n2,t),n,n);
    At2       = reshape(AmF(n2+1:2*n2,t),n,n);
    At3       = reshape(AmF(2*n2+1:3*n2,t),n,n);
    At4       = reshape(AmF(3*n2+1:4*n2,t),n,n);
    Vta       = reshape(VmF(:,t),4,4); 
    
    if projectA
        Qhte = eye(n)-Pht;
        A1   = Qhte * A1 * Qhte;
        A2   = Qhte * A2 * Qhte;
        A3   = Qhte * A3 * Qhte;
        A4   = Qhte * A4 * Qhte;

        A1   = A1 - diag(diag((A1)));
        A2   = A2 - diag(diag((A2)));
        A3   = A3 - diag(diag((A3)));
        A4   = A4 - diag(diag((A4)));
    end

    if ~Qind
        Qht       = eye(n);
    end
    
    P(:,t)    = vec(Pht);
    Q(:,t)    = vec(Qht);
    A(:,t)    = vec([vec(At1),vec(At2),vec(At3),vec(At4)]);
    V(:,t)    = vec(Vta);    
end

if dynamic
    P  = P(:,2:end);
    Q  = Q(:,2:end);
    A  = A(:,2:end);
    V  = V(:,2:end);
end

return
