function cr = mlq(param,dimpar,ulam,ulamh,ulamQ,hh,V,sigma,dynamic,n,T)
% y  = n x T matrix of outcome variables, each row contains the
       % cross-section at time t=1,...,T
% W1 = n x T d1 matrix where d1 is number of covariates in W1. Arranged as
% cov1_t=1,cov2_t=1,cov1_t=2,cov2_t=2,....
% W2 = n x T d2 matrix where d2 is number of covariates in W2. Arranged as % cov1_t=1,cov2_t=1,cov1_t=2,cov2_t=2,....
% crl: value of linear criterion function
% crq: value of quadratic criterion function


if dynamic
    lb    = 2;
    T     = T + 2;
else
    lb = 1;
    T  = T + 1;
end


d              = dimpar;
numI           = size(ulamh,2)/((1+d)*T); %number of linear moments
kq             = size(ulamQ,2)/((1+d)*T); %number of quadratic moments
ulamQp         = zeros(n*(T-1),kq);
dimV           = sqrt(size(V,1));
dim_ulam       = size(ulam,2);
dim_ulamh      = size(ulamh,2);
dim_ulamQ      = size(ulamQ,2);



if dynamic
    ulam  = [zeros(n,dim_ulam);ulam];
    ulamh = [zeros(1,dim_ulamh);ulamh];
    ulamQ = [zeros(n,dim_ulamQ);ulamQ];
    hh    = [zeros(numI^2,1),hh];
    V     = [zeros(dimV^2,1),V];
end


%allocate parameters
theta     = [1,param(1:dimpar,1)']';
f         = param(dimpar+1:end,1);
f         = [f;1]; %append normalization



%create Heap space
Pi        = zeros(T-1,T);
phi       = zeros(T,1);



%initialize phi and run recursion for phi
phi(T,1)  = 1;
for t=T-1:-1:1
    phi(t,1)   = (f(t,1)^2/sigma(t,1))+phi(t+1,1);
end

%Construct Pi forward filtering matrix



crl  = 0;
crq  = 0;

for t=lb:T-1
    Pi(t,t)        = sqrt(phi(t+1,1)/phi(t,1)/sigma(t,1));
    for s = t+1:T %construct forward filtering matrix
        Pi(t,s)    = -f(t,1)*f(s,1)*sqrt(phi(t+1,1)/phi(t,1))/(phi(t+1,1)*sqrt(sigma(t,1))*sigma(s,1));
    end
    
    ulamht          = reshape(ulamh(t,:),numI,(d+1)*T);
    crl             = crl + (ulamht*kron(theta',Pi(t,:))')'/(reshape(hh(:,t),numI,numI)) * (ulamht * kron(theta',Pi(t,:))');

    for i=1:kq
        ulamQp((t-1)*n+1:t*n,i) = ulamQ((t-1)*n+1:t*n,(i-1)*(1+d)*T+1:i*(1+d)*T)*kron(theta',Pi(t,:))';
    end
    V1              = reshape(V(:,t),dimV,dimV);
    V1              = V1(dimV-kq+1:end,dimV-kq+1:end);
    crq1            = ulamQp((t-1)*n+1:t*n,:)'*ulam((t-1)*n+1:t*n,:)*kron(theta',Pi(t,:))';
    crq             = crq + crq1' / (V1) * crq1;
end

cr  = crl + 1/(2*n) * crq;

return

